@@ -80,6 +80,7 @@ CELERY_ROUTES = {
     "pipeline.contrib.periodic_task.tasks.periodic_task_start": PIPELINE_ADDITIONAL_PRIORITY_ROUTING,
     "pipeline.contrib.periodic_task.tasks.bamboo_engine_periodic_task_start": PIPELINE_ADDITIONAL_PRIORITY_ROUTING,
     "pipeline.engine.tasks.heal_zombie_process": PIPELINE_ADDITIONAL_PRIORITY_ROUTING,
+    "pipeline.engine.tasks.expired_tasks_clean": PIPELINE_ADDITIONAL_PRIORITY_ROUTING,
     # statistics
     "pipeline.contrib.statistics.tasks.pipeline_post_save_statistics_task": PIPELINE_STATISTICS_PRIORITY_ROUTING,
     "pipeline.contrib.statistics.tasks.pipeline_archive_statistics_task": PIPELINE_STATISTICS_PRIORITY_ROUTING,
@@ -105,6 +106,11 @@ class QueueResolver(object):
 
         return self.resolve_routing_key(default_key)
 
+    def resolve_task_queue_name(self, task):
+        default_queue = self.default_setting_for(task, "queue")
+
+        return self.resolve_queue_name(default_queue)
+
     def resolve_queue_name(self, default_name):
         if not self.queue:
             return default_name
