@@ -311,7 +311,7 @@ class PipelineTemplate(models.Model):
     def subprocess_version_info(self):
         # 1. get all subprocess
         subprocess_info = TemplateRelationship.objects.get_subprocess_info(self.template_id).values(
-            "descendant_template_id", "subprocess_node_id", "version"
+            "descendant_template_id", "subprocess_node_id", "version", "always_use_latest"
         )
         info = {"subproc_has_update": False, "details": []}
         if not subprocess_info:
@@ -329,7 +329,9 @@ class PipelineTemplate(models.Model):
         for item in subprocess_info:
             item["expired"] = (
                 False
-                if item["version"] is None or item["descendant_template_id"] not in temp_current_versions
+                if item["version"] is None
+                or item["descendant_template_id"] not in temp_current_versions
+                or item["always_use_latest"]
                 else (item["version"] != temp_current_versions[item["descendant_template_id"]].current_version)
             )
             info["details"].append(item)
@@ -436,9 +438,10 @@ class TemplateRelationship(models.Model):
     """
 
     ancestor_template_id = models.CharField(_("根模板ID"), max_length=32, db_index=True)
-    descendant_template_id = models.CharField(_("子流程模板ID"), max_length=32, null=False)
+    descendant_template_id = models.CharField(_("子流程模板ID"), max_length=32, null=False, db_index=True)
     subprocess_node_id = models.CharField(_("子流程节点 ID"), max_length=32, null=False)
     version = models.CharField(_("快照字符串的md5"), max_length=32, null=False)
+    always_use_latest = models.BooleanField(_("是否永远使用最新版本"), default=False)
 
     objects = TemplateRelationShipManager()
 
@@ -598,7 +601,7 @@ class PipelineInstance(models.Model):
     )
     name = models.CharField(_("实例名称"), max_length=MAX_LEN_OF_NAME, default="default_instance")
     creator = models.CharField(_("创建者"), max_length=32, blank=True)
-    create_time = models.DateTimeField(_("创建时间"), auto_now_add=True)
+    create_time = models.DateTimeField(_("创建时间"), auto_now_add=True, db_index=True)
     executor = models.CharField(_("执行者"), max_length=32, blank=True)
     start_time = models.DateTimeField(_("启动时间"), null=True, blank=True)
     finish_time = models.DateTimeField(_("结束时间"), null=True, blank=True)
@@ -607,6 +610,7 @@ class PipelineInstance(models.Model):
     is_finished = models.BooleanField(_("是否已经完成"), default=False)
     is_revoked = models.BooleanField(_("是否已经撤销"), default=False)
     is_deleted = models.BooleanField(_("是否已经删除"), default=False, help_text=_("表示当前实例是否删除"))
+    is_expired = models.BooleanField(_("是否已经过期"), default=False, help_text=_("运行时被定期清理即为过期"))
     snapshot = models.ForeignKey(
         Snapshot,
         blank=True,
@@ -655,7 +659,7 @@ class PipelineInstance(models.Model):
     def node_id_set(self):
         if not self.tree_info:
             self.calculate_tree_info(save=True)
-        return self.tree_info.data["node_id_set"]
+        return set(self.tree_info.data["node_id_set"])
 
     @property
     def elapsed_time(self):
@@ -788,7 +792,7 @@ class PipelineInstance(models.Model):
         # get node id set
         self._get_node_id_set(node_id_set, self.execution_data)
 
-        tree_info = {"node_id_set": node_id_set}
+        tree_info = {"node_id_set": list(node_id_set)}
         self.tree_info.data = tree_info
         self.tree_info.save()
 
